#include <stdio.h>
#include <stdlib.h>

static double x; // Point of evaluation

/***********************************************************
* FUNCTION NAME: H
*
* DESCRIPTION:  recursive version of Hermite polynomial evaluator
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* int           n        I   degree of polynomial
* RETURN VALUE: double  value of Hermite polynomial on point x
*
* CHANGES: -
*************************************************************/
double H(int n)
{
    if (n == 0) return 1;
    if (n == 1) return 2 * x;
    return 2 * (n * H(n - 1) - (n - 1) * H(n - 2));
}

/***********************************************************
* FUNCTION NAME: Hnr
*
* DESCRIPTION:  non-recursive version of Hermite polynomial evaluator
*
* ARGUMENT LIST:
* Argument        Type   IO      Description
* ------------- -------- -- --------------------------------
* int           n        I   degree of polynomial
* RETURN VALUE: double  value of Hermite polynomial on point x
*
* CHANGES: -
*************************************************************/
double Hnr(int n)
{
    int i;
    int hn1; // value of H(n - 1)
    int hn2; // value of H(n - 2)
    int h; // value of Hermite polynomial on current point

    if (n == 0) return 1;
    if (n == 1) return 2 * x;
    hn1 = 2 * x;
    hn2 = 1;
    for (i = 2; i <= n; i++)
    {
        h = 2 * (i * hn1 - (i - 1) * hn2);
        hn2 = hn1;
        hn1 = h;
    }
    return h;
}


int main()
{
    int n = -1;  // Degree of Hermite polynomial
    int ok = 1;  // flag for successful conversion of x

    do
    {
        printf("Degree of Hermite polynomial [>=0]: ");
        if (1 != scanf("%d", &n))
        {
            printf("Bad value for degree\n");
            n = -1;
        }
        if (n < 0)
        {
            printf("Negative degree\n");
        }
        getchar();
    }
    while (n < 0);
    do
    {
        printf("Point of evaluation: ");
        if (1 != scanf("%lf", &x))
        {
            printf("Bad value evaluation point\n");
            ok = 0;
        }
        else ok = 1;
    }
    while (!ok);
    printf("Recursive Hermite(%d, %f)=%f\n", n, x, H(n));
    printf("Non-recursive Hermite(%d, %f)=%f\n", n, x, Hnr(n));
    return 0;
}
